﻿using System;
using System.Linq;
using Utility;
using System.IO;
using System.Threading;
using System.Diagnostics;

namespace Bll
{
    public enum ServiceState
    {
        Start,
        Stop
    }

    public class PDFtoSWF
    {
        #region 字段和属性

        private ServiceState state;

        public ServiceState State
        {
            get { return state; }
        }

        #endregion

        public PDFtoSWF()
        {
            this.state = ServiceState.Stop;
        }

        public void Start()
        {
            this.state = ServiceState.Start;

            Thread convertThread = new Thread(ConvertThread);
            convertThread.Start();

            SystemLog.CurrentLogger.Debug("服务启动成功。");
        }

        private void ConvertThread()
        {
            while (this.state != ServiceState.Stop)
            {
                try
                {
                    string pdfFile = SearchPdf();
                    if (string.IsNullOrEmpty(pdfFile))
                    {
                        Thread.Sleep(AppConfiguration.Interval * 1000);
                    }
                    else
                    {
                        ConvertPDFtoSWF(pdfFile);
                    }
                }
                catch (Exception exc)
                {
                    SystemLog.CurrentLogger.Debug("转换出现异常：\r\n" + exc.ToString());
                }
            }
            SystemLog.CurrentLogger.Debug("服务关闭成功。");
        }

        //PDF转换成SWF
        private void ConvertPDFtoSWF(string pdfFile)
        {
            using (Process p = new Process())
            {
                SystemLog.CurrentLogger.Debug(string.Format("正在处理 {0} ...", pdfFile));

                string pdf2swfExe = "pdf2swf.exe";
                string savePath = GetSavePathFromName(pdfFile);

                string cmd = pdf2swfExe;
                string args = "  -t \"" + pdfFile + "\"  -o \"" + savePath + pdfFile.Split('\\').Last().Replace(".pdf", "")
                    + "%.swf\" -s drawonlyshapes -s flashversion=9";

                p.StartInfo.FileName = cmd;
                p.StartInfo.Arguments = args;
                p.StartInfo.UseShellExecute = false;

                //此类提供的标准output流只有2k,不要重定向
                p.StartInfo.RedirectStandardOutput = false;

                p.StartInfo.CreateNoWindow = true;
                p.Start();
                p.PriorityClass = ProcessPriorityClass.High;
                p.WaitForExit();

                SystemLog.CurrentLogger.Debug(string.Format("{0} 处理完成。", pdfFile));

                if (AppConfiguration.DeleteConvertedPdf)
                {
                    //转换完成后删除Pdf
                    File.Delete(pdfFile);
                    SystemLog.CurrentLogger.Debug(string.Format("{0} 已删除。", pdfFile));
                }
                else
                {
                    //重命名Pdf
                    File.Move(pdfFile, pdfFile + ".bak");
                    SystemLog.CurrentLogger.Debug(string.Format("{0} 已重命名。", pdfFile));
                }
            }
        }

        //获取SWF存放目录
        private string GetSavePathFromName(string pdfFile)
        {
            string swfBasePath = AppConfiguration.SwfPath;
            string swfPath = swfBasePath + pdfFile.Split('\\').Last().Replace(".pdf", "") + "\\";
            if (!Directory.Exists(swfPath))
            {
                Directory.CreateDirectory(swfPath);
            }

            return swfPath;
        }

        //扫描PDF文件
        private string SearchPdf()
        {
            string pdfFile = "";

            string pdfPath = AppConfiguration.PdfPath;
            if (!Directory.Exists(pdfPath))
            {
                Directory.CreateDirectory(pdfPath);
            }

            string[] files = Directory.GetFiles(pdfPath);

            for (int i = 0; i < files.Length; i++)
            {
                if (files[i].EndsWith(".pdf"))
                {
                    pdfFile = files[i];
                    break;
                }
            }

            return pdfFile;
        }

        public void Stop()
        {
            SystemLog.CurrentLogger.Debug("服务开始关闭...");
            this.state = ServiceState.Stop;
        }
    }
}
